# -------------------------------------------------------------------------------------
# MKDoc::XML::XHTML
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      A class to linkify XHTML expressions
#
# -------------------------------------------------------------------------------------
package MKDoc::XML::XHTML;
use MKDoc::XML::Tree;
use MKDoc::Unicode;
use MKDoc::Util::Text2HTML;
use XML::Parser;
use Exporter;
use strict;
use 5.6.0;
use utf8;

our @ISA = qw /MKDoc::XML::Tree/;


##
# $self->_can_trim_spaces;
# ------------------------
#   Returns TRUE if $self ($self being a text node)
#   can be whitespace stripped
##
sub _can_trim_spaces
{
    my $self = shift;
    my $self_parent = $self->{parent};
    return 1 unless (defined $self_parent);
    
    return if ( $self->{name} eq 'code' or
		$self->{name} eq 'tt'   or
		$self->{name} eq 'samp' or
		$self->{name} eq 'kbd'  or
		$self->{name} eq 'pre'  or
		$self->{name} eq 'textarea' );
    
    return $self_parent->_can_trim_spaces;
}


##
# $self->trim_spaces;
# -------------------
#   Removes all unnecessary whitespaces,
#   carriage returns, etc
##
sub trim_spaces
{
    my $self = shift;
    if ($self->{is_text})
    {
	if ($self->_can_trim_spaces)
	{
	    my $text = $self->{text};
	    $text =~ s/(\s|\n|\r)+/ /gsm;
	    if ($text =~ /^\s*$/)
	    {
		my $previous = $self->left_sibling;
		my $next = $self->right_sibling;
		if ( defined $previous and $previous->is_inline and
		     defined $next and $next->is_inline )
		{
		    $self->{text} = ' ';
		}
		else
		{
		    $self->delete;
		}
	    }
	    else
	    {
		$self->{text} = $text;
	    }
	}
    }
    else
    {
	foreach my $child ($self->children)
	{
	    $child->trim_spaces;
	}
    }
}


##
# $self->is_inline;
# -----------------
#   Returns TRUE if the element is an inline element,
#   FALSE otherwise. This list should probably be
#   extracted from a config file at some point.
##
sub is_inline
{
    my $self = shift;
    
    # this is taken from this list
    # http://htmlhelp.com/reference/html40/olist.html
    my @list = qw /a applet basefont bdo br font iframe img
	           map area object param q script span sub sup
	           abbr acronym cite code del dfn em ins kbd
		   samp strong var
		   b big i s small strike tt u/;
    
    my $q_name = quotemeta ($self->{name});
    my $res = scalar grep (/^$q_name$/, @list);
    return $res;
}


##
# $self->text_nice;
# -----------------
#   Returns nicely indented XHTML, taking into account tags like
#   <pre>, <textarea> and <a>
##
sub text_nice
{
    my $self = shift;
    my $self_parent = $self->{parent};
    my $left_sibling = $self->left_sibling;
    
    # if this is a piece of text that has an pre, textarea
    # or inline element as a parent then we have to return it
    # untouched
    if ( defined $self_parent and
	 ( $self_parent->is_inline or
	   $self_parent->{name} eq 'pre' or
	   $self_parent->{name} eq 'textarea') and
	 ( defined $self->{is_text} and
	   $self->{is_text} ) )
    {
	return text2xml ($self->{text});
    }
    
    # if this is a piece of text an inline element as a
    # left sibling, then we return it untouched
    elsif ( $self->{is_text} and
	    defined $left_sibling and
	    $left_sibling->is_inline )
    {
	return text2xml ($self->{text});
    }
    
    # we want inline elements to be inlined!
    elsif ( $self->is_inline )
    {
	my $indent = shift;
	my $name = $self->{name};
	my $attr = $self->_attributes_text;
	my @children = $self->children;
	unless (@children)
	{
	    if ( $name eq 'br'  or $name eq 'img'   or
		 $name eq 'hr'  or $name eq 'input' or
		 $name eq 'area' )
	    {
		if ($attr eq '') { return "<$name />"       }
		else             { return "<$name $attr />" }
	    }
	    else
	    {
		if ($attr eq '') { return "<$name></$name>"       }
		else             { return "<$name $attr></$name>" }
	    }
	}
	
	my @res = ();
	if ($attr eq '') { push @res, "<$name>"       }
	else             { push @res, "<$name $attr>" }
	
	foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
	push @res, "</$name>";
	return join '', @res;
    }
    
    # pre or textarea have to be handled differently as well
    elsif ( $self->{name} eq 'pre' or
	    $self->{name} eq 'textarea' )
    {
	my $indent = shift;
	my $name = $self->{name};
	my $attr = $self->_attributes_text;
	my @children = $self->children;
	unless (@children)
	{
	    if ($attr eq '') { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name></$name>"       }
	    else             { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr></$name>" }
	}
	
	my @res = ();
	if ($attr eq '') { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name>"       }
	else             { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr>" }
	
	foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
	push @res, "</$name>";
	return join '', @res;
    }
    
    # otherwise treat everything as XML
    else
    {
	my $indent = shift;	
	if ($self->{is_text})
	{
	    return "\n" . MKDoc::XML::Tree::INDENT x $indent . text2xml ($self->{text});
	}
	
	my $name = $self->{name};
	my $attr = $self->_attributes_text;
	my @children = $self->children;
	unless (@children)
	{
	    if ( $name eq 'br'  or $name eq 'img'   or
		 $name eq 'hr'  or $name eq 'input' or
		 $name eq 'area' )
	    {
		if ($attr eq '') { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name />"       }
		else             { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr />" }
	    }
	    else
	    {
		if ($attr eq '') { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name></$name>"       }
		else             { return "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr></$name>" }
	    }
	}
	
	my @res = ();
	if ($attr eq '') { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name>"       }
	else             { push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "<$name $attr>" }
	
	foreach my $child (@children) { push @res, $child->text_nice ($indent + 1) }
	push @res, "\n" . MKDoc::XML::Tree::INDENT x $indent . "</$name>";
	return join '', @res;
    }
}


1;
