#!/usr/bin/perl
use strict;
use diagnostics;

use English;
use Pod::Usage;
use Getopt::Long;
use File::Spec::Functions qw(rel2abs);

use constant DELIMITER =>
{
  BEGIN => '[[',
  END => ']]'
};

sub getoption
{
  my %optiondefault;

  %optiondefault = (
    help => 0,
    resource => '',
    logfile => '',
    quiet => 0
  );

  GetOptions
  (
    \%optiondefault,
    'help|h',
    'resource|r=s',
    'logfile|l=s',
    'quiet|q'
  )
    or pod2usage(2);

  pod2usage(1)
    if ($optiondefault{help});

  pod2usage("ERROR: No resource file given.\n      ^")
    if (!($optiondefault{resource}) && (-t STDIN));

  return %optiondefault;
}

sub readfile
{
  my ($filename) = @_;

  # expand tildes (~), and relative paths to absolute paths
  $filename = rel2abs(glob($filename));

  open(FH, '<' . $filename)
    or die "i can't read $filename: $!\n";

  my $content = join('', <FH>);

  close(FH)
    or die "i can't close $filename: $!\n";

  return ($content);
}

sub writefile
{
  my ($filename, @content) = @_;

  # STDOUT
  if ($filename eq '')
  {
    print @content;
    return 1;
  }

  # expand tildes (~), and relative paths to absolute paths
  $filename = rel2abs(glob($filename));

#  if (-e $filename)
#  {
#    print "Warning: $filename exists. Overwrite [y|N] ?";
#    my $in = <STDIN>;
#
#    chomp($in);
#
#    return 0 if (lc($in) ne 'y');
#  }

  open(FH, '>' . $filename)
    or die "i can't write $filename: $!\n";

  print FH @content;

  close(FH)
    or die "i can't close $filename: $!\n";

  return 1;
}

sub unique
{
  my ($resource, $quiet) = @_;

  my $BEGIN = DELIMITER->{BEGIN};
  my $END = DELIMITER->{END};

  my ($convert, %capture, @warning);
  my $duplicate = 0;

  $convert = readfile($resource);

  while ($convert =~
    /
    (
    \Q$BEGIN\E # catch the start delimiter
    .*?        # grab the contents, and don't be greedy
    \Q$END\E   # catch the end delimiter
    )

    .*?        # allow anything between key and translation

    (
    \Q$BEGIN\E # catch the start delimiter
    .*?        # grab the contents
    \Q$END\E   # catch the end delimiter
    )
    /gsmx
  )
  {
    my ($key, $value) = ($1, $2);
    my %by_value;

    if (exists $capture{$key} && ($capture{$key} eq $value))
    {
      $duplicate++;
      print join ("\n",
        'Note: Duplicate found and removed.',
        "Resource: $resource",
        "Key: $key",
        "Translation: $value\n\n")
        if (!$quiet);
      next;
    }

    push @warning, join ("\n",
      'Warning: Duplicate key with different translations. Using second (later) translation.',
      "Resource: $resource",
      "Duplicate Key: $key",
      "First translation : $capture{$key}",
      "Second translation: $value\n\n")
      if (exists $capture{$key});

    while (my ($key2, $value2) = each %capture)
    {
      $by_value{$value2} = $key2;
    }

    push @warning, join ("\n",
      'Warning: Duplicate translation with differing keys. Using both.',
      "Resource: $resource",
      "Duplicate translation: $value",
      "First key : $by_value{$value}",
      "Second key: $key\n\n")
      if (exists $by_value{$value} && ($value ne (DELIMITER->{BEGIN} . DELIMITER->{END})));

    $capture{$key} = $value;

    print join ("\n",
      'Note: Unique key and translation found.',
      "Resource: $resource",
      "Key: $key",
      "Translation: $value\n\n")
      if (!$quiet);
  }


  my @fix = '';
  foreach my $key (sort keys %capture)
  {
    push @fix, "$key\n$capture{$key}\n\n";
  }

  writefile($resource, @fix);
  print "$duplicate duplicate(s) found/removed.\n\n";

  return (@warning);
}

sub main
{
  my (%option, @warn);

  %option = getoption();
  @warn = unique($option{resource}, $option{quiet});
  writefile($option{logfile}, @warn);
}

main();

__END__

=pod

=head1 NAME

mkuniq.pl - Remove duplicate entries from a resource file.

=head1 SYNOPSIS

Usage: mkuniq.pl --resource=<file> [options]

WARNING: The resource file is overwritten - use a copy if you want to go back and use the logs to make things better.

=head1 EXAMPLE

mkuniq.pl -r xx.html

Try --help for more help.

=head1 OPTIONS

    -h   --help                  print this message
    -r   --resource=<file>       mkdoc resource file
    -l   --logfile=<file>        a file to log warnings to
    -q   --quiet                 suppress non-critical error messages

=over 8

=head1 DESCRIPTION

Duplicate key/translation pairs are discarded while mismatched pairs are flagged:

- if the same key is found with a different translation, then the first translation
is lost and the later used.

- if the same translation is found with different keys, then both keys and
translations are used (although there's something redundant going on).

=item -h --help

Print a brief help message and exits.

=item -r --resource=<file>

Specify which MKDoc resource file to strip duplicates from.

=item -l --logfile=<file>

Specify a log file for warning messages.

=item -q --quiet

Suppress non-critical messages.

=back

=cut

